--------------------------------------------------------------------------------
-- Copyright (c) 1995-2003 Xilinx, Inc.
-- All Right Reserved.
--------------------------------------------------------------------------------
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /    Vendor: Xilinx 
-- \   \   \/     Version : 7.1.04i
--  \   \         Application : ISE WebPACK
--  /   /         Filename : cfg_ser_in_tb.vhw
-- /___/   /\     Timestamp : Sun Jan 22 23:58:46 2006
-- \   \  /  \ 
--  \___\/\___\ 
--
--Command: 
--Design Name: cfg_ser_in_tb
--Device: Xilinx
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY cfg_ser_in_tb IS
END cfg_ser_in_tb;

ARCHITECTURE testbench_arch OF cfg_ser_in_tb IS
    COMPONENT cfg_ser_in
        PORT (
            CFGCLK : In std_logic;
            RESET : In std_logic;
            SDATA_IN : In std_logic;
            SDATA_ENABLE : In std_logic;
            DATA_OUT : Out std_logic_vector (15 DownTo 0);
            DATA_RDY : Out std_logic
        );
    END COMPONENT;

    SIGNAL CFGCLK : std_logic := '0';
    SIGNAL RESET : std_logic := '1';
    SIGNAL SDATA_IN : std_logic := '0';
    SIGNAL SDATA_ENABLE : std_logic := '0';
    SIGNAL DATA_OUT : std_logic_vector (15 DownTo 0) := "0000000000000000";
    SIGNAL DATA_RDY : std_logic := '00000000000000000000000000000000';

    SHARED VARIABLE TX_ERROR : INTEGER := 0;
    SHARED VARIABLE TX_OUT : LINE;

    constant PERIOD : time := 100 ns;
    constant DUTY_CYCLE : real := 0.5;
    constant OFFSET : time := 0 ns;

    BEGIN
        UUT : cfg_ser_in
        PORT MAP (
            CFGCLK => CFGCLK,
            RESET => RESET,
            SDATA_IN => SDATA_IN,
            SDATA_ENABLE => SDATA_ENABLE,
            DATA_OUT => DATA_OUT,
            DATA_RDY => DATA_RDY
        );

        PROCESS    -- clock process for CFGCLK
        BEGIN
            WAIT for OFFSET;
            CLOCK_LOOP : LOOP
                CFGCLK <= '0';
                WAIT FOR (PERIOD - (PERIOD * DUTY_CYCLE));
                CFGCLK <= '1';
                WAIT FOR (PERIOD * DUTY_CYCLE);
            END LOOP CLOCK_LOOP;
        END PROCESS;

        PROCESS
            PROCEDURE CHECK_DATA_OUT(
                next_DATA_OUT : std_logic_vector (15 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (DATA_OUT /= next_DATA_OUT) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns DATA_OUT="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, DATA_OUT);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_DATA_OUT);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_DATA_RDY(
                next_DATA_RDY : std_logic;
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (DATA_RDY /= next_DATA_RDY) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns DATA_RDY="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, DATA_RDY);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_DATA_RDY);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            BEGIN
                -- -------------  Current Time:  20ns
                WAIT FOR 20 ns;
                RESET <= '0';
                SDATA_ENABLE <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  40ns
                WAIT FOR 20 ns;
                SDATA_IN <= '1';
                SDATA_ENABLE <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  60ns
                WAIT FOR 20 ns;
                SDATA_IN <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  80ns
                WAIT FOR 20 ns;
                SDATA_IN <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  120ns
                WAIT FOR 40 ns;
                SDATA_IN <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  140ns
                WAIT FOR 20 ns;
                SDATA_IN <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  160ns
                WAIT FOR 20 ns;
                SDATA_IN <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  180ns
                WAIT FOR 20 ns;
                SDATA_IN <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  200ns
                WAIT FOR 20 ns;
                SDATA_IN <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  240ns
                WAIT FOR 40 ns;
                SDATA_IN <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  260ns
                WAIT FOR 20 ns;
                SDATA_IN <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  280ns
                WAIT FOR 20 ns;
                SDATA_ENABLE <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  300ns
                WAIT FOR 20 ns;
                SDATA_IN <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  320ns
                WAIT FOR 20 ns;
                SDATA_IN <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  340ns
                WAIT FOR 20 ns;
                RESET <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  400ns
                WAIT FOR 60 ns;
                RESET <= '0';
                WAIT FOR 100 ns;

                IF (TX_ERROR = 0) THEN
                    STD.TEXTIO.write(TX_OUT, string'("No errors or warnings"));
                    ASSERT (FALSE) REPORT
                      "Simulation successful (not a failure).  No problems detected."
                      SEVERITY FAILURE;
                ELSE
                    STD.TEXTIO.write(TX_OUT, TX_ERROR);
                    STD.TEXTIO.write(TX_OUT,
                        string'(" errors found in simulation"));
                    ASSERT (FALSE) REPORT "Errors found during simulation"
                         SEVERITY FAILURE;
                END IF;
            END PROCESS;

    END testbench_arch;

